from Products.Archetypes.public import BaseFolderSchema, Schema
from Products.Archetypes.public import StringField, LinesField, IntegerField, BooleanField
from Products.Archetypes.public import SelectionWidget, LinesWidget, TextAreaWidget, MultiSelectionWidget, IntegerWidget, RichWidget, IdWidget, StringWidget, BooleanWidget
from Products.Archetypes.public import BaseFolder, registerType
from Products.CMFCore import CMFCorePermissions
from DateTime import DateTime
from Products.SimpleBlog.config import DISPLAY_MODE
import Products.SimpleBlog.Permissions

from Products.ATContentTypes.content.base import ATCTFolder

schema = ATCTFolder.schema.copy() +  Schema((
    StringField('description',
                isMetadata=1,
                accessor='Description',
                searchable=1,                
                widget=TextAreaWidget(label='Description', 
                                      label_msgid="label_blog_description",
                                      description_msgid="help_blog_description",
                                      i18n_domain="SimpleBlog",
                                      description='Give a description for this SimpleBlog.')),
    # this field is deprecated, here for compatibility
    StringField('displayMode',
                vocabulary=DISPLAY_MODE,
                widget=SelectionWidget(label='Display Mode', 
                                       label_msgid="label_display_mode",
                                       description_msgid="help_display_mode",
                                       i18n_domain="SimpleBlog",
                                       description='Choose the display mode.',
                                       visible={'view' : 'invisible', 'edit':'invisible'}),
                default='descriptionOnly'),
    IntegerField('displayItems', 
                widget=IntegerWidget(label='BlogEntries to display', 
                                      label_msgid="label_display_items",
                                      description_msgid="help_display_items",
                                      i18n_domain="SimpleBlog",
                                      description='Set the maximum number of BlogEntries to display.'), 
                 default=20),
    BooleanField('warnForUnpublishedEntries', 
             default=1, 
             widget=BooleanWidget(label='Show unpublished entries warning', 
                        i18n_domain="SimpleBlog",
                        label_msgid="label_warnForUnpublishedEntries",
                        description_msgid="help_warnForUnpublishedEntries",
                        description='When checked, a warning will be displayed on the blog\'s frontpage if there are entries that are not yet published.')),
    BooleanField('allowCrossPosting', 
             default=1, 
             widget=BooleanWidget(label='Allow cross-posting', 
                        i18n_domain="SimpleBlog",
                        label_msgid="label_allowCrossPosting",
                        description_msgid="help_allowCrossPosting",
                        description='When checked, this blog will include cross-post entries from other blogs.')),
    LinesField('categories', 
               widget=LinesWidget(label='Possible Categories', 
                                  label_msgid="label_categories",
                                  description_msgid="help_categories",
                                  i18n_domain="SimpleBlog",
                                  description='Supply the list of possible categories that can be used in SimpleBlog Entries.'))
        ))

# hide relatedItems
schema['relatedItems'].widget.visible={'view' : 'invisible', 'edit':'invisible'}

class Blog(ATCTFolder):
    """Blog"""

    portal_type = meta_type = 'Blog'
    archetype_name = 'Blog'
    content_icon='simpleblog_icon.gif'
    schema = schema
    global_allow=1
    
    default_view = 'simpleblog_view_title_description'
    immediate_view = 'simpleblog_view_title_description'
    suppl_views = ('simpleblog_view_title_description', 'simpleblog_view_title', 'simpleblog_view_title_description_body')

    filter_content_types=1    
    allowed_content_types=('BlogEntry', 'BlogFolder', 'Link', 'Image', 'File', 'Portlet')

    def canSetDefaultPage(self):
        return False


    def manage_afterAdd(self, item, container):
        BaseFolder.manage_afterAdd(self, item, container)
        if self.simpleblog_tool.getCreatePortletOnBlogCreation():
            if not hasattr(item.aq_base, 'right_slots'):
                item._setProperty('right_slots', ['here/portlet_simpleblog/macros/portlet'], 'lines')

    def synContentValues(self):
        # get brains for items that are published within the context of this blog.
        entries = self.simpleblog_tool.searchForEntries(self, maxResults=0)
        
        # convert to objects
        objs = [e.getObject() for e in entries]
        return objs
        
    def listCategories(self):
        cats=self.getCategories()
           
        # add the global categories
        for c in self.simpleblog_tool.getGlobalCategories():
            if not c in cats:
                cats.append(c)           
        cats = list(cats)
        cats.sort()
        return tuple(cats)                

    def getForeignEntries(self):
        """ Returns all entries from other blogs that are published into this blog using the remoteBlogs value """
        return [f for f in self.getBRefs('AppearsIn') if self.portal_membership.checkPermission('View', f)]

        
    def getEntries(self, category=None, maxResults=None, fromHere=0, filterState=1, sort=1, join=0, addCrossPostInfo=0, **kwargs):
        """ Return all the contained published entries, real objects, not the brains """
        # see simpleblog_tool.searchForEntries for API description

        query=kwargs

        publishedState = self.simpleblog_tool.getPublishedState()
        
        if category!=None:
            query['EntryCategory']=category

        query['getAlwaysOnTop']=1
        
        if filterState:
            query['review_state']=publishedState            
    
        # first the items that need to be listed on top
        localOnTop = self.portal_catalog.searchResults(query, meta_type='BlogEntry', path={'query':self.simpleblog_tool.getObjectPath(self),'level':0}, sort_order='reverse', sort_on='effective')
        localOnTop = [r.getObject() for r in localOnTop ]
        
        # then the other items
        query['getAlwaysOnTop']=0
        localNoTop = self.portal_catalog.searchResults(query, meta_type='BlogEntry', path={'query':self.simpleblog_tool.getObjectPath(self),'level':0}, sort_order='reverse', sort_on='effective')
        localNoTop= [r.getObject() for r in localNoTop]        
        
        # foreign items
        if self.getAllowCrossPosting():
            foreignEntries = self.getForeignEntries()
        else:
            foreignEntries=[]

        # filter out the always on top entries
        foreignOnTop = [e for e in foreignEntries if e.getAlwaysOnTop()]
        foreignNoTop = [e for e in foreignEntries if not e.getAlwaysOnTop()]
        
        # so, now we have:
        # total = localOnTop + foreignOnTop + localNoTop + foreignNoTop
        # and that needs to be sorted
        

        if addCrossPostInfo:
            # make each object a tuple (obj, <iscrosspost>)
            foreignOnTop = [(e,1) for e in foreignOnTop]
            foreignNoTop = [(e,1) for e in foreignNoTop]
            localOnTop = [(e,0) for e in localOnTop]
            localNoTop = [(e,0) for e in localNoTop]
        
        onTop = foreignOnTop + localOnTop
        onBottom = foreignNoTop + localNoTop
        
        if sort and foreignEntries:
            if addCrossPostInfo:
                onTop.sort((lambda x,y:cmp(y[0].effective(), x[0].effective())))
                onBottom.sort((lambda x,y:cmp(y[0].effective(), x[0].effective())))
            else:
                onTop.sort((lambda x,y:cmp(y.effective(), x.effective())))
                onBottom.sort((lambda x,y:cmp(y.effective(), x.effective())))
                      
        if join:
            results = onTop+onBottom
            if maxResults==0:
                return results
            elif maxResults==None:
                return results[:self.simpleblog_tool.getMaxItemsInPortlet()]
            else:
                return results[:maxResults]               
        else:
            return (onTop, onBottom)
        
registerType(Blog)

